SystemOrganization addCategory: #'Control-Core'!
SystemOrganization addCategory: #'Control-Exceptions'!

!BlockClosure methodsFor: '*Control' stamp: 'fbs 4/12/2011 07:58'!
reset
	"Mark the call stack. The handler says 'Exception, this is where the nearest reset is."
	^ self on: DelimitedControlException do: [:e | e resume: thisContext home].! !

!BlockClosure methodsFor: '*Control' stamp: 'fbs 4/13/2011 07:28'!
shift
	"I am a unary block. Take the call stack to the nearest reset, reifying that partial continuation as a function F. Then evaluate myself with F as an argument.
	Note that this is a convenience method for Object>>shift:. See that method's comments for details."
	^ self shift: self.
! !

!BlockClosure methodsFor: '*Control' stamp: 'fbs 4/14/2011 15:18'!
shift: aUnaryBlock
	"Turn the call stack down to the nearest #reset, and reify this partial continuation as a function. Then, pass this function to aUnaryBlock."

	| currentCtxt fun resetCtxt |
	currentCtxt := thisContext.
	resetCtxt := DelimitedControlException signal.
	resetCtxt ifNil: [^ MissingResetException signal].
	
	"Store the stack from _below_ the shift to just _above_ the reset."
	fun := PartialContinuation from: currentCtxt sender downTo: resetCtxt sender.

	"Throw away the stack down to the context that called the reset."
	currentCtxt swapSender: resetCtxt sender.

	^ aUnaryBlock value: fun.! !

Error subclass: #MissingResetException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Exceptions'!

Notification subclass: #DelimitedControlException
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Exceptions'!

!DelimitedControlException commentStamp: 'fbs 3/31/2011 17:39' prior: 0!
I mark a context so that a shift further up the stack will unwind only to me. In other words, use me to delimit a continuation.!

Object subclass: #PartialContinuation
	instanceVariableNames: 'values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Control-Core'!

!PartialContinuation commentStamp: 'fbs 4/12/2011 18:58' prior: 0!
I am a slice of a call stack, that is to say, some sequence of ContextParts. Use me like a Block by calling my #value or #value: methods.

My serialisation/deserialisation code comes straight from Seaside's WAPartialContinuation.!

!PartialContinuation class methodsFor: 'instance creation' stamp: 'fbs 4/12/2011 19:11'!
from: topContext downTo: bottomContext
	^ self new from: topContext downTo: bottomContext.! !

!PartialContinuation methodsFor: 'private' stamp: 'fbs 4/15/2011 07:55'!
deserialiseFrom: anObject
	| stream context |
	stream := anObject readStream.
	"We don't need to explicitly connect the contexts, because one of the instvars is sender."
	[stream atEnd] whileFalse:
		[context := stream next.
		1 to: context class instSize do: [ :index | context instVarAt: index put: (stream next) ].
		1 to: context size do: [ :index | context at: index put: (stream next) ] ].
	
	"Return our bottom-most context."
	^ context! !

!PartialContinuation methodsFor: 'initialize-release' stamp: 'fbs 4/13/2011 07:32'!
from: topContext downTo: bottomContext
	values := self serialiseFrom: topContext to: bottomContext.! !

!PartialContinuation methodsFor: 'accessing' stamp: 'fbs 4/14/2011 15:41'!
numArgs
	^ 1! !

!PartialContinuation methodsFor: 'private' stamp: 'fbs 4/14/2011 15:20'!
serialiseFrom: aContext to: lowerContext
	| c stream |
	stream := WriteStream on: (OrderedCollection new: 100).
"	Transcript showln: '8<-----'."
	c := aContext.
	[c isNil or: [c == lowerContext]] whileFalse:
		["Transcript showln: c printString."
		stream nextPut: c.
		1 to: c class instSize do: [ :index | stream nextPut: (c instVarAt: index) ].
		1 to: c size do: [ :index | stream nextPut: (c at: index) ].
		c := c sender.].
	c ifNil: [MissingResetException signal: lowerContext printString , ' not found while capturing continuation'].
	
	^ stream contents.! !

!PartialContinuation methodsFor: 'accessing' stamp: 'fbs 4/14/2011 07:35'!
stack
	"A debugging aid. Return a Collection containing my contexts."
	| result stream context |
	result := OrderedCollection new.
	stream := values readStream.

	[stream atEnd] whileFalse:
		[context := stream next.
		1 to: context class instSize do: [ :index | stream next ].
		1 to: context size do: [ :index | stream next ].
		result addLast: context].
	
	^ result.! !

!PartialContinuation methodsFor: 'evaluating' stamp: 'fbs 4/12/2011 18:59'!
value
	"Splice my contents onto the stack, and execute me."
	self value: nil.! !

!PartialContinuation methodsFor: 'evaluating' stamp: 'fbs 4/18/2011 22:03'!
value: anObject
	"Deserialise the continuation, returning the first/bottom-most context. Stitch the continuation onto the current call stack by telling this bottom context that thisContext's sender called it. 'thisContext sender' because thisContext refers to the context of the #value: call."
	(self deserialiseFrom: values) swapSender: thisContext sender.
	
	"Then, tell thisContext that its sender was actually the last/top-most context."
	thisContext swapSender: values first.
	
	"Finally, 'return' anObject from the caller context... which thanks to the line
	above, is actually values first. This effectively sets anObject as the parameter
	to the reified partial continuation, because instead of 'returning' to our caller
	we 'return' to the next part of the computation - our
	partial-continuation-as-function."

	^ anObject.! !
